@@ -33,12 +33,16 @@ from .tools import _get_dump_files
 from .tools import _binary_zip
 from .tools import autocleanpaper
 from .tools import _shell_complete_file
+from .tools import print_prod_env
 from .cli import cli, pass_config, Commands
 from .lib_clickhelpers import AliasedGroup
 from .tools import ensure_project_name
 from .tools import _get_filestore_folder
 from .tools import __try_to_set_owner
 from .tools import docker_list_containers
+from .tools import __get_postgres_volume_name
+from .tools import get_volume_fullpath
+from .tools import force_input_hostname
 
 import inspect
 import os
@@ -66,6 +70,20 @@ def restore(config):
     pass
 
 
+from .tools import get_directory_size
+
+
+@backup.command()
+@pass_config
+@click.pass_context
+def used_space_files(ctx, config):
+    filestore_folder = _get_filestore_folder(config)
+
+    size = int(get_directory_size(filestore_folder))
+    print("----")
+    print(size)
+
+
 @backup.command(name="all")
 @click.argument("filename", required=False)
 @pass_config
@@ -272,7 +290,7 @@ def _restore_wodoo_bin(ctx, config, filepath, verify):
                 "docker",
                 "volume",
                 "inspect",
-                f"{config.PROJECT_NAME}_odoo_postgres_volume",
+                __get_postgres_volume_name(config),
             ],
             encoding="utf-8",
         )
@@ -396,6 +414,8 @@ def restore_db(
     if not (dbname or config.dbname):
         raise Exception("somehow dbname is missing")
 
+    print_prod_env(config)
+
     dumps_path = config.dumps_path
     BACKUPDIR = Path(dumps_path)
     filename_absolute = (BACKUPDIR / filename).absolute()
@@ -404,6 +424,9 @@ def restore_db(
     if not config.force:
         __restore_check(filename_absolute, config)
 
+    if config.force and not config.devmode:
+        force_input_hostname()
+
     params = {
         "no_dev_scripts": no_dev_scripts,
         "no_remove_webassets": no_remove_webassets,
@@ -498,7 +521,9 @@ def _restore_dump(
         with config.forced() as config:
             _dropdb(config, conn)
         try_ignore_exceptions(
-            lambda: dropdb(config), (psycopg2.errors.AdminShutdown, psycopg2.InterfaceError), timeout=30
+            lambda: dropdb(config),
+            (psycopg2.errors.AdminShutdown, psycopg2.InterfaceError),
+            timeout=30,
         )
 
     def create_db():
@@ -687,7 +712,7 @@ def _backup_wodoobin(ctx, config, filename):
                 "docker",
                 "volume",
                 "inspect",
-                f"{config.PROJECT_NAME}_odoo_postgres_volume",
+                __get_postgres_volume_name(config),
             ]
         )
     )[0]["Mountpoint"]
