@@ -1,5 +1,5 @@
 import traceback
-from .tools import bash_find
+from .tools import bashfind
 import arrow
 import threading
 from tabulate import tabulate
@@ -38,6 +38,11 @@ from .odoo_config import MANIFEST
 from .tools import execute_script
 from .tools import ensure_project_name
 
+import inspect
+import os
+from pathlib import Path
+current_dir = Path(os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe()))))
+
 
 @cli.group(cls=AliasedGroup)
 @pass_config
@@ -80,11 +85,15 @@ def config(ctx, config, service_name, full=True):
 
 
 def _get_arch():
-    arch = subprocess.check_output(["uname", "-m"], encoding="UTF-8").strip()
-    return {
+    mapping = {
         "x86_64": "amd64",
         "aarch64": "arm64",
-    }[arch]
+    }
+    arch = subprocess.check_output(["uname", "-m"], encoding="UTF-8").strip()
+    if arch in mapping:
+        return mapping[arch]
+    else:
+        return arch
 
 
 @composer.command(
@@ -102,7 +111,7 @@ def _get_arch():
 @click.option("-cR", "--additional_config_raw", help="like ODOO_DEMO=1;RUN_PROXY=0")
 @click.option("--images-url", help="default: https://github.com/marcwimmer/odoo")
 @click.option("-I", "--no-update-images", is_flag=True)
-@click.option("-A", "--no-auto-repo", is_flag=True)
+@click.option("--no-gimera-apply", is_flag=True)
 @click.option(
     "--docker-compose", help="additional docker-compose files, separated by colon :"
 )
@@ -121,10 +130,11 @@ def do_reload(
     additional_config_raw,
     images_url,
     no_update_images,
-    no_auto_repo,
     docker_compose,
+    no_gimera_apply,
 ):
     from .myconfigparser import MyConfigParser
+    from .module_tools import NotInAddonsPath
 
     if headless and proxy_port:
         click.secho(("Proxy Port and headless together not compatible."), fg="red")
@@ -171,19 +181,22 @@ def do_reload(
         if docker_compose:
             docker_compose = docker_compose.split(":")
 
-        internal_reload(
-            ctx,
-            config,
-            db,
-            demo,
-            devmode,
-            headless,
-            proxy_port,
-            mailclient_gui_port,
-            additional_config,
-            apply_auto_repo=not no_auto_repo,
-            additional_docker_configuration_files=docker_compose,
-        )
+        try:
+            internal_reload(
+                ctx,
+                config,
+                db,
+                demo,
+                devmode,
+                headless,
+                proxy_port,
+                mailclient_gui_port,
+                additional_config,
+                additional_docker_configuration_files=docker_compose,
+                no_gimera_apply=no_gimera_apply,
+            )
+        except NotInAddonsPath as ex:
+            abort(str(ex))
 
     finally:
         if additional_config_file and additional_config_file.exists():
@@ -205,8 +218,8 @@ def internal_reload(
     proxy_port,
     mailclient_gui_port,
     additional_config=None,
-    apply_auto_repo=True,
     additional_docker_configuration_files=None,
+    no_gimera_apply=False,
 ):
     ensure_project_name(config)
     additional_docker_configuration_files = additional_docker_configuration_files or []
@@ -243,13 +256,16 @@ def internal_reload(
 
         click.secho("Additional config: {defaults}")
 
-    if apply_auto_repo:
-        _apply_autorepo(ctx=ctx, config=config)
+    ODOO_VERSION = str(MANIFEST()["version"]).split(".")[0]
+    KEY_ODOO_VERSION = f"RUN_ODOO_VERSION_{ODOO_VERSION}"
+    os.environ[KEY_ODOO_VERSION] = ODOO_VERSION
+    defaults[KEY_ODOO_VERSION] = ODOO_VERSION
 
     # assuming we are in the odoo directory
     _do_compose(
         **defaults,
         additional_docker_configuration_files=additional_docker_configuration_files,
+        no_gimera_apply=no_gimera_apply,
     )
 
     _execute_after_reload(config)
@@ -274,6 +290,7 @@ def _do_compose(
     db="",
     demo=False,
     additional_docker_configuration_files=None,
+    no_gimera_apply=False,
     **forced_values,
 ):
     """
@@ -516,7 +533,7 @@ def _prepare_yml_files_from_template_files(
         if dir.is_file():
             _files += [dir]
         else:
-            for file in bash_find(dir, "docker-compose*.yml"):
+            for file in bashfind(dir, "docker-compose*.yml"):
                 _files.append(file)
 
     if config.restrict and config.restrict.get("docker-compose"):
@@ -671,19 +688,24 @@ def __run_docker_compose_config(config, contents, env):
         files = []
         for i, content in enumerate(contents):
             file_path = temp_path / f"docker-compose-{str(i).zfill(5)}.yml"
-            file_path.write_text(yaml.dump(content, default_flow_style=False))
+            file_content = yaml.dump(content, default_flow_style=False)
+            file_path.write_text(file_content)
             files.append(file_path)
             del file_path
 
-        cmdline = [
-            str(config.files["docker_compose_bin"]),
-        ]
-        for file_path in files:
-            cmdline += [
-                "-f",
-                file_path,
+        def buildcmd(files):
+            cmdline = [
+                str(config.files["docker_compose_bin"]),
             ]
-        cmdline += ["config"]
+            for file_path in files:
+                cmdline += [
+                    "-f",
+                    file_path,
+                ]
+            cmdline += ["config"]
+            return cmdline
+
+        cmdline = buildcmd(files)
         d = deepcopy(os.environ)
         d.update(env)
 
@@ -693,14 +715,22 @@ def __run_docker_compose_config(config, contents, env):
         except KeyError:
             d["DOCKER_GROUP_ID"] = "0"
 
-        (temp_path / 'cmd').write_text(" ".join(map(str, cmdline)))
+        (temp_path / "cmd").write_text(" ".join(map(str, cmdline)))
         try:
             conf = subprocess.check_output(cmdline, cwd=temp_path, env=d)
         except:
-            for file in temp_path.glob("*"):
-                click.secho(f"{file}:\n", fg='yellow')
-                click.secho(file.read_text())
-            # get the output
+            # find culprit
+            for i in range(len(files)):
+                file = files[i]
+                cmdline2 = buildcmd(files[: i + 1])
+                try:
+                    click.secho(f"Testing {file}...", fg="green")
+                    conf = subprocess.check_output(cmdline2, cwd=temp_path, env=d)
+                except:
+                    click.secho(f"{file}:\n", fg="yellow")
+                    click.secho(file.read_text())
+                    break
+
             conf = subprocess.check_output(cmdline, cwd=temp_path, env=d)
             sys.exit(-1)
         conf = yaml.safe_load(conf)
@@ -925,7 +955,9 @@ def _use_file(config, path):
         if "NO-AUTO-COMPOSE" in path.read_text():
             return False
         if "images" in path.parts:
-            if not getattr(config, "run_{}".format(path.parent.name)):
+            if not getattr(config, f"run_{path.parent.name}") and not any(
+                ".run_" in x for x in path.parts
+            ):
                 return False
             if not any(".run_" in x for x in path.parts):
                 # allower postgres/docker-compose.yml
@@ -938,9 +970,6 @@ def _use_file(config, path):
             run_key = "RUN_{}".format(path.parent.name).upper()
             return getattr(config, run_key)
 
-        if f"run_odoo_version.{config.odoo_version}.yml" in path.name:
-            return True
-
         customs_dir = config.customs_dir
         try:
             path.relative_to(customs_dir)
@@ -951,7 +980,7 @@ def _use_file(config, path):
                 return True
 
         # requires general run:
-        if getattr(config, "run_{}".format(path.parent.name)) or "run_" in path.name:
+        if getattr(config, f"run_{path.parent.name}") or "run_" in path.name:
             run = list(
                 filter(
                     lambda x: x.startswith("run_"),
@@ -991,18 +1020,6 @@ def _use_file(config, path):
     return res
 
 
-def _apply_autorepo(ctx, config):
-    from .odoo_config import MANIFEST_CLASS, customs_dir
-
-    manifest = MANIFEST_CLASS()
-    if not manifest.get("auto_repo", False):
-        return
-
-    from .lib_src import _turn_into_odoosh
-
-    _turn_into_odoosh(ctx, customs_dir())
-
-
 def _merge_odoo_dockerfile(config):
     """
     If customs contains dockerfile, then append their execution
@@ -1033,15 +1050,82 @@ def _merge_odoo_dockerfile(config):
     if dockerfile1:
         click.secho(f"Copying {dockerfile1} to {config.files['odoo_docker_file']}")
         config.files["odoo_docker_file"].write_text(Path(dockerfile1).read_text())
-        for file in bash_find(config.WORKING_DIR, "Dockerfile.appendix"):
+        for file in bashfind(config.WORKING_DIR, "Dockerfile.appendix"):
             appendix = file.read_text()
             file = config.files["odoo_docker_file"]
             content = file.read_text() + "\n" + appendix
             content = content.replace("${PROJECT_NAME}", config.project_name)
             file.write_text(content)
+
+        # include source code
+        if not config.SRC_EXTRA:
+            import pudb;pudb.set_trace()
+            append_odoo_src(config, config.files["odoo_docker_file"])
+
     else:
         # seems to use images from registry; no build section
         pass
 
 
+def append_odoo_src(config, path):
+    content = "ADD {src} /opt/src"
+    path.write_text(path.read_text() + "\n" + content)
+
+def _complete_setting_name(ctx, param, incomplete):
+    lines = (current_dir / 'settings.txt').read_text().splitlines()
+
+    params = []
+    for line in lines:
+        if not line.strip():
+            continue
+        try:
+            name, doc = line.replace("\t", " ").split(" ", 1)
+        except ValueError:
+            name = line
+            doc = ""
+        name = name.strip()
+        doc = doc.strip()
+        name = name.split("=")[0]
+        params.append({'name': name, 'doc': doc})
+
+
+    res = set([x['name'].strip() for x in params])
+    if incomplete:
+        res = list(filter(lambda x: x.lower().startswith(incomplete.lower()), res))
+    return sorted(res)
+
+@composer.command()
+@pass_config
+@click.pass_context
+@click.argument("name", required=False, shell_complete=_complete_setting_name)
+@click.argument("value", required=False, default="")
+def setting(ctx, config, name, value):
+    from .myconfigparser import MyConfigParser
+    if not name:
+        ctx.invoke(show_effective_settings)
+        return
+    configparser = MyConfigParser(config.files["settings"])
+    if '=' in name:
+        name, value = name.split("=", 1)
+    if not value:
+        for k in sorted(configparser.keys()):
+            if  name.lower() in k.lower():
+                click.secho(f"{k}={configparser[k]}")
+    else:
+        configparser = MyConfigParser(config.files['project_settings'])
+        configparser[name] = value
+        configparser.write()
+        click.secho(f"{name}={value}", fg='green')
+        ctx.invoke(do_reload)
+
+@composer.command()
+@pass_config
+@click.pass_context
+def show_effective_settings(ctx, config):
+    from .myconfigparser import MyConfigParser
+
+    config = MyConfigParser(config.files["settings"])
+    for k in sorted(config.keys()):
+        click.echo(f"{k}={config[k]}")
+
 Commands.register(do_reload, "reload")
