@@ -6,11 +6,13 @@ import os
 from .cli import cli, pass_config, Commands
 from .lib_clickhelpers import AliasedGroup
 from .tools import execute_script
+from .tools import force_input_hostname
 import subprocess
 import json
 from pathlib import Path
 from .tools import abort
 from .tools import ensure_project_name
+from .tools import print_prod_env
 
 
 @cli.group(cls=AliasedGroup)
@@ -31,14 +33,13 @@ def pull(ctx, config):
 
 @docker.command()
 @click.option("-b", "--build", is_flag=True)
-@click.option("-k", "--kill", is_flag=True)
 @pass_config
 @click.pass_context
-def dev(ctx, config, build, kill):
+def dev(ctx, config, build):
     ensure_project_name(config)
     if config.use_docker:
         from .lib_control_with_docker import dev as lib_dev
-    return lib_dev(ctx, config, build, kill)
+    return lib_dev(ctx, config, build, kill=True)
 
 
 @docker.command(name="ps")
@@ -95,7 +96,8 @@ def remove_volumes(ctx, config, dry_run):
         click.secho(f"Removing: {vol}", fg="red")
         if not dry_run:
             rc = subprocess.run(
-                ["docker", "volume", "rm", "-f", vol], encoding="utf8",
+                ["docker", "volume", "rm", "-f", vol],
+                encoding="utf8",
                 capture_output=True,
             )
             if rc.returncode:
@@ -182,6 +184,11 @@ def down(ctx, config, machines, volumes, remove_orphans, postgres_volume):
     from .lib_control_with_docker import down as lib_down
     from .lib_db_snapshots_docker_zfs import NotZFS
 
+    print_prod_env(config)
+
+    if not config.devmode and volumes:
+        force_input_hostname()
+
     if postgres_volume or volumes:
         if postgres_volume:
             if not config.force:
@@ -256,28 +263,32 @@ def attach(ctx, config, machine):
 @click.option("--pull", is_flag=True)
 @click.option("--push", is_flag=True)
 @click.option("-p", "--plain", is_flag=True)
+@click.option("-s", "--include-source", is_flag=True)
 @pass_config
 @click.pass_context
-def build(ctx, config, machines, pull, no_cache, push, plain):
+def build(ctx, config, machines, pull, no_cache, push, plain, include_source):
     ensure_project_name(config)
     if plain:
         os.environ["BUILDKIT_PROGRESS"] = "plain"
     from .lib_control_with_docker import build as lib_build
 
-    lib_build(ctx, config, machines, pull, no_cache, push)
+    lib_build(ctx, config, machines, pull, no_cache, push, include_source)
 
 
 @docker.command()
 @click.argument("machine", required=True)
 @click.option("-c", "--command", required=False, help="Like /odoolib/debug.py")
 @click.option("-p", "--ports", is_flag=True, help="With Port 33284")
+@click.option("--port", help="Define the debug port")
 @pass_config
 @click.pass_context
-def debug(ctx, config, machine, ports, command):
+def debug(ctx, config, machine, ports, command, port):
     ensure_project_name(config)
     from .lib_control_with_docker import debug as lib_debug
+    if port:
+        ports = int(port)
 
-    lib_debug(ctx, config, machine, ports, cmd=command)
+    lib_debug(ctx, config, machine, ports=port, cmd=command)
 
 
 @cli.command()
@@ -326,6 +337,8 @@ def logall(config, machines, follow, lines):
 )
 @pass_config
 def shell(config, command, queuejobs):
+    print_prod_env(config)
+
     ensure_project_name(config)
     command = "\n".join(command)
     from .lib_control_with_docker import shell as lib_shell
@@ -495,6 +508,8 @@ def transfer_volume_content(context, config, show_all, filter, no_backup):
     )
 
 
+
+
 Commands.register(run)
 Commands.register(runbash)
 Commands.register(do_kill, "kill")
