@@ -16,6 +16,7 @@ from .cli import cli, pass_config, Commands
 from .lib_clickhelpers import AliasedGroup
 from .tools import __hash_odoo_password
 from .tools import _make_sure_module_is_installed
+from .tools import print_prod_env
 
 
 @cli.group(cls=AliasedGroup)
@@ -94,6 +95,8 @@ def pgactivity(config):
 def pgcli(config, dbname, params, host, port, user, password):
     from .tools import DBConnection
 
+    print_prod_env(config)
+
     dbname = dbname or config.dbname
 
     if host:
@@ -106,7 +109,6 @@ def pgcli(config, dbname, params, host, port, user, password):
         conn = config.get_odoo_conn(inside_container=True).clone(dbname=dbname)
     return _pgcli(config, conn, params, use_docker_container=True)
 
-
 @db.command()
 @click.argument("dbname", required=False)
 @click.argument("params", nargs=-1)
@@ -177,10 +179,10 @@ def _pgcli(config, conn, params, use_docker_container=None):
 @db.command(name="reset-odoo-db")
 @click.argument("dbname", required=False)
 @click.option("--do-not-install-base", is_flag=True)
-@click.option("-C", "--collateC", is_flag=True, help="Use Collate C like odoo suggests")
 @pass_config
 @click.pass_context
-def reset_db(ctx, config, dbname, do_not_install_base, collatec):
+def reset_db(ctx, config, dbname, do_not_install_base):
+    collatec = True
     dbname = dbname or config.dbname
     if not dbname:
         raise Exception("dbname required")
@@ -235,10 +237,10 @@ def anonymize(ctx, config):
 
 
 @db.command()
-@click.option("--no-update", is_flag=True)
+@click.option("-V", "--no-vacuum-full", is_flag=True)
 @pass_config
 @click.pass_context
-def cleardb(ctx, config, no_update):
+def cleardb(ctx, config, no_vacuum_full):
     if not (config.devmode or config.force):
         click.secho("Either DEVMODE or force required", fg="red")
         sys.exit(-1)
@@ -246,36 +248,13 @@ def cleardb(ctx, config, no_update):
     _make_sure_module_is_installed(
         ctx, config, "cleardb", "https://github.com/marcwimmer/odoo-cleardb.git"
     )
-
-    if not no_update:
-        Commands.invoke(
-            ctx,
-            "update",
-            module=["cleardb"],
-            no_restart=False,
-            no_dangling_check=True,
-            no_update_module_list=False,
-            non_interactive=True,
-            recover_view_error=True,
-        )
-
-        # update of all modules then required, so that metainformation is
-        # written to ir.model (the _cleardb flag on model)
-        Commands.invoke(
-            ctx,
-            "update",
-            module=[],
-            no_restart=False,
-            no_dangling_check=True,
-            no_update_module_list=False,
-            non_interactive=True,
-        )
+    str_no_vauum_full = "1" if no_vacuum_full else "0"
 
     Commands.invoke(
         ctx,
         "odoo-shell",
         command=[
-            'env["frameworktools.cleardb"]._run()',
+            f'env["frameworktools.cleardb"]._run(no_vacuum_full={str_no_vauum_full})',
             "env.cr.commit()",
         ],
     )
