@@ -2,13 +2,8 @@ import arrow
 import sys
 import click
 import inquirer
-from .tools import remove_webassets
-from .tools import _askcontinue
-from .tools import get_volume_names
 from .cli import cli, pass_config, Commands
 from .lib_clickhelpers import AliasedGroup
-from .tools import __hash_odoo_password
-from .tools import _remove_postgres_connections, _execute_sql
 from .tools import get_filesystem_of_folder
 
 
