@@ -29,6 +29,7 @@ from .tools import __concurrent_safe_write_file
 from .cli import cli, pass_config, Commands
 from .lib_clickhelpers import AliasedGroup
 from .tools import _execute_sql
+from .tools import table_exists
 from .tools import get_services
 from .tools import __try_to_set_owner
 from .tools import measure_time, abort
@@ -62,11 +63,9 @@ def odoo_module(config):
 @pass_config
 def abort_upgrade(config):
     click.echo("Aborting upgrade...")
-    SQL = """
-        UPDATE ir_module_module SET state = 'installed' WHERE state = 'to upgrade';
-        UPDATE ir_module_module SET state = 'uninstalled' WHERE state = 'to install';
-    """
-    _execute_sql(config.get_odoo_conn(), SQL)
+    from .module_tools import Modules, DBModules
+
+    DBModules.abort_upgrade()
 
 
 def _get_default_modules_to_update():
@@ -270,7 +269,7 @@ def _get_outdated_versioned_modules_of_deptree(modules):
                 # seen: if version in manifest is 14.0.1.0 and installed in
                 # 13.0 odoo then version becomes inside odoo: 13.0.14.0.10
                 # so try to match including current version:
-                odoo_version = list(map(int, str(MANIFEST()['version']).split(".")))
+                odoo_version = list(map(int, str(MANIFEST()["version"]).split(".")))
                 new_version = tuple(odoo_version + list(new_version))
                 if new_version != version:
                     yield dep
@@ -450,13 +449,22 @@ def update(
 
     To update all (custom) modules set "all" here
 
+    Sample call migration 13.0 -> 14.0:
+    OPTS="--no-dangling-check --config-file=config_migration --server-wide-modules=web,openupgrade_framework --additional-addons-paths=openupgrade"
+    odoo update $OPTS base
 
-    Sample call migration 14.0:
-    odoo update --no-dangling-check --config-file=config_migration --server-wide-modules=web,openupgrade_framework --additional-addons-paths=openupgrade base
-
+    # Real world sample:
+    odoo update $OPTS base
+    odoo update $OPTS mail
+    odoo psql --sql "delete from ir_ui_view where id in (1,2,3,4);"
+    odoo update $OPTS web
+    odoo update $OPTS sale_product_configurator
+    odoo update $OPTS hr
+    odoo update $OPTS stock
 
 
     """
+    started = arrow.get()
 
     param_module = module
 
@@ -655,7 +663,7 @@ def update(
     )
 
     if uninstall or all_modules:
-        _uninstall_marked_modules(config, MANIFEST().get("uninstall", []))
+        _uninstall_marked_modules(ctx, config, MANIFEST().get("uninstall", []))
 
     # check danglings
     if not no_dangling_check and all_modules:
@@ -666,6 +674,10 @@ def update(
 
     _set_sha(config)
 
+    duration = (arrow.get() - started).total_seconds()
+    date = arrow.get().strftime("%Y-%m-%d %H:%M:%S")
+    click.secho(f"Update done at {date} - duration {duration}s", fg="yellow")
+
 
 def _set_sha(config):
     conn = config.get_odoo_conn()
@@ -788,10 +800,10 @@ def _parse_modules(modules):
 @click.pass_context
 def uninstall(ctx, config, modules):
     modules = _parse_modules(modules)
-    _uninstall_marked_modules(config, modules)
+    _uninstall_marked_modules(ctx, config, modules)
 
 
-def _uninstall_marked_modules(config, modules):
+def _uninstall_marked_modules(ctx, config, modules):
     """
     Checks for file "uninstall" in customs root and sets modules to uninstalled.
     """
@@ -812,6 +824,7 @@ def _uninstall_marked_modules(config, modules):
     if config.use_docker:
         from .lib_control_with_docker import shell as lib_shell
 
+    uninstalled = False
     for module in modules:
         click.secho(f"Uninstall {module}", fg="red")
         lib_shell(
@@ -826,6 +839,10 @@ def _uninstall_marked_modules(config, modules):
             ),
         )
         del module
+        uninstalled = True
+
+    if uninstalled:
+        Commands.invoke(ctx, "restart", machines=["odoo"])
 
     modules = [x for x in modules if DBModules.is_module_installed(x)]
     if modules:
@@ -1307,22 +1324,18 @@ def _get_changed_files(git_sha):
         if submodule:
             current_commit = str(repo.hex)
             relpath = filepath.relative_to(repo.path)
-            old_commit = (
-                subprocess.check_output(
-                    [
-                        "git",
-                        "rev-parse",
-                        f"{git_sha}:./{relpath}",
-                    ], encoding='utf8'
-                )
-                .strip()
-            )
-            new_commit = (
-                subprocess.check_output(
-                    ["git", "rev-parse", f"{current_commit}:./{relpath}"], encoding='utf8',
-                )
-                .strip()
-            )
+            old_commit = subprocess.check_output(
+                [
+                    "git",
+                    "rev-parse",
+                    f"{git_sha}:./{relpath}",
+                ],
+                encoding="utf8",
+            ).strip()
+            new_commit = subprocess.check_output(
+                ["git", "rev-parse", f"{current_commit}:./{relpath}"],
+                encoding="utf8",
+            ).strip()
             # now diff the submodule
             for filepath2 in git_diff_files(filepath, old_commit, new_commit):
                 filepaths2.append(str(relpath / filepath2))
@@ -1371,20 +1384,6 @@ def _get_global_hash_paths(relative_to_customs_dir=False):
     return [p.relative_to(customs_dir_path) for p in global_hash_paths]
 
 
-def _clean_customs(ctx, config):
-    from .odoo_config import customs_dir
-
-    path = customs_dir()
-    if not config.force:
-        abort(
-            "Needs force option, because I call git clean -xdff and "
-            "all your work is lost. (Stashing before)"
-        )
-    if not is_git_clean(path):
-        subprocess.check_call(["git", "stash", "--include-untracked"], cwd=path)
-    subprocess.check_call(["git", "clean", "-xdff"], cwd=path)
-
-
 hash_cache = {}
 
 
@@ -1408,8 +1407,6 @@ def list_deps(ctx, config, module, no_cache):
     from .odoo_config import customs_dir
     from .consts import FILE_DIRHASHES
 
-    _clean_customs(ctx, config)
-
     click.secho("Loading Modules...", fg="yellow")
     modules = Modules()
     if module == "all":
@@ -1624,5 +1621,30 @@ def list_installed_modules(config, fix_not_in_manifest, only_customs):
         manifest.rewrite()
 
 
+@odoo_module.command()
+@click.argument("module", required=True, shell_complete=_get_available_modules)
+@click.pass_context
+@pass_config
+def zip(config, ctx, module):
+    from .module_tools import Modules, Module
+    from .odoo_config import customs_dir
+
+    try:
+        mod = Module.get_by_name(module)
+    except:
+        module_path = Path(os.getcwd()) / module
+
+    if not module_path.exists():
+        abort(f"{module_path} not found.")
+
+    zipfile = Path(os.getcwd()) / f"{module}.zip"
+    if zipfile.exists():
+        zipfile.unlink()
+    subprocess.check_call(
+        ["/usr/bin/zip", "-r", zipfile.name, "."], cwd=module_path
+    )
+
+
+
 Commands.register(update)
 Commands.register(show_install_state)
