@@ -1,11 +1,12 @@
+import time
 from pathlib import Path
+from queue import Queue
+import threading
 from click.shell_completion import CompletionItem
 import json
 import yaml
 import shutil
 import subprocess
-import inquirer
-import sys
 from datetime import datetime
 import os
 import click
@@ -24,6 +25,10 @@ from .tools import bashfind
 from .tools import cwd
 from .tools import __rmtree
 from .tools import _get_customs_root
+from .tools import _get_xml_id
+from .tools import pretty_xml
+from .tools import _execute_sql
+
 
 ADDONS_OCA = "addons_OCA"
 
@@ -34,79 +39,6 @@ def src(config):
     pass
 
 
-@click.command()
-@click.pass_context
-@pass_config
-def ensure_odoosh_repo(config, ctx):
-    _ensure_odoosh_repo(ctx)
-
-
-def _ensure_odoosh_repo(ctx):
-    from gimera.gimera import apply as gimera_apply
-
-    odoosh_path = Path(os.environ["ODOOSH_REPO"] or "../odoo.sh").resolve().absolute()
-    if not odoosh_path.exists():
-        subprocess.check_call(
-            [
-                "git",
-                "clone",
-                "https://github.com/Odoo-Ninjas/odoo.sh.git",
-                odoosh_path,
-            ]
-        )
-        with cwd(odoosh_path.absolute()):
-            ctx.invoke(gimera_apply)
-    return odoosh_path
-
-
-def _build_gimera(path):
-    gimera_file = path / "gimera.yml"
-    if gimera_file.exists():
-        content = yaml.safe_load(gimera_file.read_text())
-    else:
-        content = {
-            "repos": [
-                {
-                    "branch": str(current_version()),
-                    "url": "https://github.com/odoo/odoo",
-                    "path": "odoo",
-                    "type": "integrated",
-                },
-                {
-                    "branch": str(current_version()),
-                    "url": "git@github.com:odoo/enterprise",
-                    "path": "enterprise",
-                    "type": "integrated",
-                },
-                {
-                    "branch": str(current_version()),
-                    "url": "git@github.com:odoo/design-themes",
-                    "path": "themes",
-                    "type": "integrated",
-                },
-            ]
-        }
-        for repo in content["repos"]:
-            __assure_gitignore(customs_dir() / ".gitignore", "/" + repo["path"] + "/")
-    current_content = gimera_file.read_text() if gimera_file.exists() else ""
-    new_content = yaml.dump(content, default_flow_style=False)
-
-    content_changed = False
-    if current_content != new_content:
-        gimera_file.write_text(new_content)
-        content_changed = True
-    return content_changed
-
-
-def _turn_into_odoosh(ctx, path):
-    _ensure_odoosh_repo(ctx)
-    content_changed = _build_gimera(path)
-
-    repos = yaml.safe_load((path / "gimera.yml").read_text())
-    _apply_gimera_if_required(ctx, path, repos, force_do=content_changed)
-    _find_duplicate_modules()
-
-
 def _find_duplicate_modules():
     from .module_tools import Modules
 
@@ -118,20 +50,22 @@ def _find_duplicate_modules():
 def _apply_gimera_if_required(ctx, path, content, force_do=False):
     from gimera.gimera import apply as gimera
 
-    def needs_apply():
+    with cwd(path):
         for repo in content["repos"]:
             repo_path = path / repo["path"]
-            if not repo_path.exists():
-                return True
-        return False
+            if repo["type"] == "submodule" or force_do or not repo_path.exists():
+                ctx.invoke(
+                    gimera, repos=[repo["path"]], recursive=True, no_auto_commit=True
+                )
+                changed = True
+        else:
+            changed = False
 
-    if force_do or needs_apply():
-        with cwd(path):
-            ctx.invoke(gimera, recursive=True)
+        if changed:
             click.secho(
                 "Restarting reloading because gimera apply was done", fg="yellow"
             )
-            Commands.invoke(ctx, "reload", no_auto_repo=True)
+            Commands.invoke(ctx, "reload", no_apply_gimera=True)
 
             from .module_tools import Modules
 
@@ -144,11 +78,6 @@ def _apply_gimera_if_required(ctx, path, content, force_do=False):
 def apply_gimera_if_required(ctx):
     path = customs_dir()
     gimera_file = path / "gimera.yml"
-    if not gimera_file.exists():
-        if MANIFEST().get("auto_repo", False):
-            _build_gimera(path)
-        else:
-            return
     repos = yaml.safe_load(gimera_file.read_text())
     _apply_gimera_if_required(ctx, path, repos)
 
@@ -162,25 +91,19 @@ def find_duplicate_modules(config, ctx):
 
 @src.command(name="init", help="Create a new odoo")
 @click.argument("path", required=True)
+@click.argument("version", required=False)
 @click.option("--odoosh", is_flag=True)
 @click.pass_context
 @pass_config
-def init(config, ctx, path, odoosh):
+def init(config, ctx, path, odoosh, version):
     from .module_tools import make_customs
 
     path = Path(path)
+    path = path.absolute()
     if not path.exists():
         path.mkdir(parents=True)
-    make_customs(ctx, path)
-
-    odoosh and _turn_into_odoosh(ctx, Path(os.getcwd()))
-
-
-@src.command(help="Makes odoo and enterprise code available from common code")
-@click.pass_context
-@pass_config
-def make_odoo_sh_compatible(config, ctx):
-    _turn_into_odoosh(ctx, customs_dir())
+    os.chdir(path)
+    make_customs(config, ctx, path, version, odoosh)
 
 
 @src.command()
@@ -237,7 +160,7 @@ def show_addons_paths():
 @pass_config
 def make_modules(config):
     modules = ",".join(MANIFEST()["install"])
-    (config.dirs["customs"] / "modules.txt").write_text(modules)
+    (customs_dir() / "modules.txt").write_text(modules)
     click.secho(f"Updated /modules.txt with: \n\n", fg="yellow")
     click.secho(modules)
 
@@ -471,7 +394,6 @@ def _identify_duplicate_modules(check):
                     )
 
 
-
 @src.command(name="pretty-print-manifest")
 def pretty_print_manifest():
     from .odoo_config import MANIFEST
@@ -498,6 +420,7 @@ def security(config, module, model):
     # give rights to choose
     # TODO ...
 
+
 @src.command()
 @click.option("-d", "--dry-run", is_flag=True)
 @click.pass_context
@@ -505,6 +428,7 @@ def security(config, module, model):
 def delete_modules_not_in_manifest(config, ctx, dry_run):
     from .module_tools import Modules, Module
     from .odoo_config import customs_dir
+
     modules = Modules()
     all_modules = modules.modules
     installed_modules = list(sorted(modules.get_all_used_modules()))
@@ -513,11 +437,255 @@ def delete_modules_not_in_manifest(config, ctx, dry_run):
     for mod in all_modules:
         if mod not in installed_modules:
             mod = Module.get_by_name(mod)
-            if not any(str(mod.path).startswith(X) for X in [
-                'odoo',
-                'odoo/odoo',
-                'enterprise',
-                'themes',
-            ]):
-                click.secho(f"Deleting: {mod.path}", fg='red')
-                shutil.rmtree(root / mod.path)
\ No newline at end of file
+            if not any(
+                str(mod.path).startswith(X)
+                for X in [
+                    "odoo",
+                    "odoo/odoo",
+                    "enterprise",
+                    "themes",
+                ]
+            ):
+                click.secho(f"Deleting: {mod.path}", fg="red")
+                shutil.rmtree(root / mod.path)
+
+
+@src.command()
+@click.pass_context
+@pass_config
+def grab_views(config, ctx):
+    root = customs_dir() / "src" / "views"
+
+    sql = f"select id, name, arch_db, model from ir_ui_view"
+    conn = config.get_odoo_conn()
+    rows = _execute_sql(conn, sql, fetchall=True)
+
+    all_files = []
+    if root.exists():
+        all_files = [x for x in list(root.glob("**/*")) if not x.is_dir()]
+
+    threads = []
+    stats = {"views": 0}
+    q = Queue()
+
+    def check_view():
+        while not q.empty():
+            count, view = q.get()
+            try:
+
+                def prog(c):
+                    p = round(count / len(rows) * 100, 1)
+                    click.secho(f"...threading progress {p}%", fg="yellow")
+
+                arch = view[2]
+                xmlid = _get_xml_id(config, "ir.ui.view", view[0])
+                model = view[3] or "no-model"
+                name = view[1]
+                id = view[0]
+                if current_version() < 16:
+                    arch = {"en_US": arch}
+
+                for key in arch:
+                    if xmlid:
+                        filepath = root / f"{model}.xmlid.{xmlid}.xml"
+                    else:
+                        filepath = (
+                            root / "by_name" / f"{model}.byname.{name}.{key}.{id}.xml"
+                        )
+                    filepath.parent.mkdir(exist_ok=True, parents=True)
+
+                    path = filepath.parent / (filepath.stem + f".{key}.xml")
+                    xml = arch[key].encode("utf8")
+                    path.write_bytes(pretty_xml(xml))
+                    if path in all_files:
+                        all_files.remove(path)
+                stats["views"] += 1
+                if not stats["views"] % 800:
+                    prog(stats["views"])
+            except Exception as ex:
+                click.secho(ex, fg="red")
+
+    threads = []
+    for count, view in enumerate(rows):
+        q.put((count, view))
+
+    for i in range(10):
+        t = threading.Thread(target=check_view)
+        t.daemon = True
+        t.start()
+        threads.append(t)
+    [x.join() for x in threads]
+
+    click.secho(f"Exported: {stats['views']} views.", fg="green")
+    if all_files:
+        for file in all_files:
+            file.unlink()
+            click.secho(f"View does not exist anymore: {file.relative_to(root)}")
+
+
+@src.command()
+@click.option("-t", "--threads", default=10)
+@click.argument("match", required=False)
+@click.pass_context
+@pass_config
+def compare_views(config, ctx, threads, match):
+    root = customs_dir() / "src"
+
+    q = Queue()
+
+    conn = config.get_odoo_conn()
+
+    click.secho(
+        'name="%(project_task_action_from_partner)d muss ersetzt werden', fg="red"
+    )
+    time.sleep(5)
+
+    def compare_view(file_content, res_id, lang, info, the_model):
+        view = _execute_sql(
+            conn, f"select arch_db from ir_ui_view where id={res_id}", fetchone=True
+        )
+        if not view:
+            click.secho(f"VIEW vanished with id: {res_id} ({info})", fg="red")
+        else:
+            view = view[0]
+            if current_version() < 16:
+                view = {"en_US": view}
+            for _lang, _arch in view.items():
+                if _lang == lang:
+
+                    def strippi(xml):
+                        xml = xml.splitlines()
+                        if "<?" in xml[0]:
+                            xml = "\n".join(xml[1:])
+                        else:
+                            xml = "\n".join(xml)
+                        for c in [" \t\n"]:
+                            xml = xml.replace(c, "")
+                        return xml
+
+                    if strippi(file_content) != strippi(_arch):
+                        click.secho(
+                            f"Model: {the_model} -----------------------------------------------------------",
+                            fg="yellow",
+                        )
+                        click.secho(f"VIEW {res_id} {info}", fg="green")
+                        Path("/tmp/1").write_bytes(
+                            pretty_xml(file_content.encode("utf8"))
+                        )
+                        Path("/tmp/2").write_bytes(pretty_xml(_arch.encode("utf8")))
+                        subprocess.run(
+                            ["/bin/diff", "--color", "-w", "/tmp/1", "/tmp/2"]
+                        )
+
+    conn = config.get_odoo_conn()
+
+    athreads = []
+
+    def check_file():
+        while not q.empty():
+            file = q.get()
+            if match and match not in file.stem:
+                continue
+            try:
+                content = file.read_text()
+                if ".xmlid." in file.stem:
+                    try:
+                        module, name, lang = file.stem.split(".xmlid.")[1].split(".", 4)
+                        model = file.stem.split(".xmlid.")[0]
+                    except:
+                        click.secho(
+                            f"Invalid File Format: {file.relative_to(root)}", fg="red"
+                        )
+                        return
+                    xmlid = ".".join([module, name])
+
+                    sql = (
+                        f"select res_id "
+                        f"from ir_model_data "
+                        f"where model = 'ir.ui.view' "
+                        f"and module='{module}' "
+                        f"and name = '{name}'"
+                    )
+                    row = _execute_sql(conn, sql, fetchone=True)
+                    if not row:
+                        click.secho(f"XMLID vanished: {xmlid}", fg="red")
+                    else:
+                        arch = file.read_text()
+                        compare_view(content, row[0], lang, xmlid, model)
+                    del xmlid, module, name
+                else:
+                    content = file.read_text()
+                    model = file.stem.split(".byname.")[0]
+                    res_id = int(file.stem.split(".")[-2])
+                    lang = file.stem.split(".")[-3]
+                    compare_view(content, res_id, lang, "", model)
+                    del res_id
+            except Exception as ex:
+                click.secho(ex, fg="red")
+
+    for file in (customs_dir() / "src" / "views").glob("*.xml"):
+        q.put(file)
+
+    folder = customs_dir() / "src" / "views" / "by_name"
+    if folder.exists():
+        for file in folder.glob("*"):
+            q.put(file)
+
+    for _ in range(threads):
+        t = threading.Thread(target=check_file)
+        t.start()
+        athreads.append(t)
+    [t.join() for t in athreads]
+
+
+@src.command()
+@click.pass_context
+@pass_config
+def grab_models(config, ctx):
+    root = customs_dir() / "src" / "models"
+
+    sql = f"select id, model from ir_model"
+    conn = config.get_odoo_conn()
+    rows = _execute_sql(conn, sql, fetchall=True)
+    q = Queue()
+
+    for i, (id, model) in enumerate(rows):
+        q.put(model)
+
+    def do():
+        while not q.empty():
+            model = q.get()
+            try:
+                data = {
+                    "model": model,
+                    "fields": [],
+                }
+                fields = _execute_sql(
+                    conn,
+                    f"select name, ttype, compute, relation, translate, readonly from ir_model_fields where model = '{model}'",
+                    fetchall=True,
+                )
+                for field in fields:
+                    data["fields"].append(
+                        {
+                            "name": field[0],
+                            "type": field[1],
+                            "compute": field[2],
+                            "relation": field[3],
+                            "translate": field[4],
+                            "readonly": field[5],
+                        }
+                    )
+                path = root / (model + ".json")
+                path.parent.mkdir(exist_ok=True, parents=True)
+                path.write_text(json.dumps(data, indent=4))
+
+            except Exception as ex:
+                click.secho("{e}", fg="red")
+
+    threads = []
+    for _ in range(10):
+        t = threading.Thread(target=do)
+        t.start()
+        threads.append(t)
+    [t.join() for t in threads]
