@@ -1,19 +1,14 @@
-from pathlib import Path
 import json
-import subprocess
-import inquirer
-import sys
 from datetime import datetime
-import os
+from pathlib import Path
+
 import click
-from .odoo_config import current_version
-from .odoo_config import MANIFEST
-from .odoo_config import customs_dir
-from .cli import cli, pass_config
-from .lib_clickhelpers import AliasedGroup
-from .tools import _execute_sql
 from tabulate import tabulate
-from .cli import cli, pass_config, Commands
+
+from .cli import Commands, cli, pass_config
+from .lib_clickhelpers import AliasedGroup
+from .odoo_config import MANIFEST, current_version, customs_dir
+from .tools import _execute_sql
 
 
 @cli.group(cls=AliasedGroup)
@@ -168,7 +163,7 @@ def progress(config):
 @talk.command()
 @pass_config
 def modules_overview(config):
-    from .module_tools import Modules, Module
+    from .module_tools import Module, Modules
 
     modules = Modules()
 
@@ -264,6 +259,7 @@ def menus(config, name):
     cols = list(rows[0])[:2]
     cols.insert(0, "xmlid")
     cols.insert(0, "path")
+    tablerows = list(sorted(tablerows, key=lambda x: x[0]))
     click.secho(tabulate(tablerows, cols, tablefmt="fancy_grid"), fg="yellow")
 
 
@@ -315,7 +311,7 @@ def users(config, login):
     rows = _execute_sql(
         conn,
         sql=(
-            f"SELECT login, name FROM res_users INNER JOIN "
+            f"SELECT res_users.id as user_id, login, name FROM res_users INNER JOIN "
             f"res_partner p on p.id = res_users.partner_id "
             f"WHERE p.name ILIKE '%{login}%' or login ILIKE '%{login}%'"
         ),
