@@ -18,6 +18,8 @@ from .tools import whoami
 from .tools import abort
 from .tools import __rmtree as rmtree
 from .tools import pretty_xml
+from .tools import bashfind
+from .tools import __assure_gitignore
 
 try:
     from psycopg2 import IntegrityError
@@ -26,6 +28,7 @@ except Exception:
 from .tools import _extract_python_libname
 from .tools import _exists_table
 from .tools import _execute_sql
+from .tools import table_exists
 from .tools import measure_time
 from .odoo_config import get_conn_autoclose, manifest_file_names
 from .odoo_config import current_version
@@ -148,10 +151,21 @@ def get_all_langs(config):
     return langs
 
 
-def get_modules_from_install_file(include_uninstall):
+def get_modules_from_install_file(include_uninstall=False):
     res = MANIFEST().get("install", [])
     if include_uninstall:
-        res += MANIFEST().get("uninstall", [])
+        for mod in MANIFEST().get("uninstall", []):
+            try:
+                Module.get_by_name(mod)
+            except (NotInAddonsPath, Module.IsNot, KeyError):
+                click.secho(
+                    f"WARNING: module {mod} cannot be uninstalled - "
+                    "not found in source",
+                    fg="yellow",
+                )
+                pass
+            else:
+                res += [mod]
     return res
 
 
@@ -172,7 +186,8 @@ class DBModules(object):
             UPDATE ir_module_module SET state = 'uninstalled' WHERE state = 'to install';
         """
         with get_conn_autoclose() as cr:
-            _execute_sql(cr, SQL)
+            if table_exists(cr, 'ir_module_module'):
+                _execute_sql(cr, SQL)
 
     @classmethod
     def show_install_state(clazz, raise_error):
@@ -330,7 +345,7 @@ class DBModules(object):
             return state[1] in ["installed", "to upgrade"]
 
 
-def make_customs(ctx, path):
+def make_customs(config, ctx, path, version, odoosh):
     from gimera.gimera import apply as gimera
     from .tools import abort
     import click
@@ -338,7 +353,10 @@ def make_customs(ctx, path):
     if not path.exists():
         abort("Path does not exist: {}".format(path))
     elif list(path.glob("*")):
-        abort("Path is not empty: {}".format(path))
+        files = [x for x in path.glob("*") if x.name != '.git']
+        if files:
+            if not config.force:
+                abort("Path is not empty: {}".format(path))
 
     import inquirer
     from .tools import copy_dir_contents
@@ -355,39 +373,30 @@ def make_customs(ctx, path):
     versions = sorted(
         [x.name for x in src_dir.glob("*")], key=lambda x: _floatify(x), reverse=True
     )
-    version = inquirer.prompt([inquirer.List("version", "", choices=versions)])[
-        "version"
-    ]
-    del versions
+    if not version:
+        version = inquirer.prompt([inquirer.List("version", "", choices=versions)])[
+            "version"
+        ]
+        del versions
 
     copy_dir_contents(src_dir / version, path)
 
     manifest_file = path / "MANIFEST"
     manifest = eval(manifest_file.read_text())
-    raise Exception("Rewrite to use gimera")
 
-    click.echo("Checking for odoo repo at env variable 'ODOO_REPO'")
-    if os.getenv("ODOO_REPO", ""):
-        odoo_path = path / "odoo"
-        repo_path = Path(os.environ["ODOO_REPO"])
-        repo = Repo(repo_path)
-        repo.X("git", "checkout", str(version))
-        odoo_path.mkdir()
-        sha = repo.hex
-        click.echo("Copying odoo with sha to local directory [{}]".format(sha))
-        copy_dir_contents(repo_path, odoo_path, exclude=[".git"])
-        manifest["odoo_commit"] = sha
-
-    manifest_file.write_text(json.dumps(manifest, indent=4))
-
-    subprocess.call(["git", "init"], cwd=path)
+    if not (path / ".git").exists():
+        subprocess.call(["git", "init"], cwd=path)
+    for repo in ["odoo", "enterprise", "themes"]:
+        __assure_gitignore(path / ".gitignore", "/" + repo + "/")
     subprocess.call(["git", "add", "."], cwd=path)
     subprocess.call(["git", "commit", "-am", "init"], cwd=path)
     ctx.invoke(gimera, recursive=True, update=True)
     try_to_set_owner(whoami(), path)
+    subprocess.call(["odoo reload"], shell=True, cwd=path)
 
     click.secho("Initialized - please call following now.", fg="green")
-    click.secho("odoo db reset", fg="green")
+    click.secho("odoo next   (to get a free next port)", fg="green")
+    click.secho("odoo -f db reset", fg="green")
     sys.exit(0)
 
 
@@ -660,13 +669,8 @@ class Modules(object):
             Returns a list of full paths of all manifests
             """
             for path in reversed(get_odoo_addons_paths()):
-                mans = subprocess.check_output(
-                    ["find", ".", "-maxdepth", "2", "-name", manifest_file_names()],
-                    cwd=path,
-                    encoding="utf8",
-                ).splitlines()
+                mans = bashfind(path, name=manifest_file_names(), maxdepth=2)
                 for file in sorted(mans):
-                    file = path / file
                     modname = file.parent.name
                     if modname in modnames:
                         continue
@@ -758,7 +762,7 @@ class Modules(object):
                     continue
                 try:
                     dep_mod = Module.get_by_name(dep, no_deptree=True)
-                except (NotInAddonsPath, Module.IsNot):
+                except (NotInAddonsPath, Module.IsNot, KeyError):
                     # if it is a module, which is probably just auto install
                     # but not in the manifest, then it is not critical
                     if dep not in remark_about_missing_module_info:
@@ -930,12 +934,7 @@ class Modules(object):
         # keep highest version and or leaveout loosers
         def _map(x):
             if x:
-                try:
-                    arr = iscompatible.parse_requirements(x)
-                except:
-                    import pudb
-
-                    pudb.set_trace()
+                arr = iscompatible.parse_requirements(x)
             else:
                 arr = []
             for arr in arr:
@@ -1129,7 +1128,7 @@ class Module(object):
                 self._manifest_dict = eval(content)  # TODO safe
 
             except (SyntaxError, Exception) as e:
-                abort((f"error at file: {self.manifest_path}:\n{str(e)}"), fg="red")
+                abort(f"error at file: {self.manifest_path}:\n{str(e)}")
         return self._manifest_dict
 
     def __make_path_relative(self, path):
@@ -1184,6 +1183,13 @@ class Module(object):
                 path = dir
             del dir
         if not path:
+            possible_matches = bashfind(".", name=name, type="d")
+            if possible_matches:
+                click.secho("Found the missing module here:", fg="yellow", bold=True)
+                for dir in possible_matches:
+                    click.secho(dir, fg="yellow")
+                click.secho("Please add it to the manifest addons-paths")
+
             raise NotInAddonsPath(f"Could not get path for {name}")
         if path.exists():
             path = path.resolve()
@@ -1198,7 +1204,10 @@ class Module(object):
         # could be an odoo module then
         for path in get_odoo_addons_paths():
             if (path / name).resolve().is_dir():
-                return Module(path / name)
+                try:
+                    return Module(path / name)
+                except Module.IsNot:
+                    pass
         raise KeyError(f"Module not found or not linked: {name}")
 
     @property
@@ -1341,6 +1350,8 @@ class Module(object):
             to_remove.getparent().remove(to_remove)
 
         if current_version() > 14.0:
+            if filepath.exists():
+                filepath.unlink()
             manifest = self.path / "__manifest__.py"
             jsoncontent = eval(manifest.read_text())
             jsoncontent.setdefault("assets", {})
