@@ -1,3 +1,4 @@
+import subprocess
 from pathlib import Path
 import os
 import re
@@ -641,6 +642,8 @@ def update_cache(arg_modified_filename=None):
     finally:
         f.close()
 
+    subprocess.run(f"sort -o {plainfile} -u {plainfile}", shell=True)
+
     return plainfile
 
 
