@@ -1,3 +1,4 @@
+import socket
 import uuid
 import time
 from subprocess import PIPE, STDOUT
@@ -167,6 +168,17 @@ def __get_odoo_commit():
     return commit
 
 
+def table_exists(conn, table):
+    exist = _execute_sql(
+        conn,
+        f"select count(*) from information_schema.tables where table_schema='public' and table_name='{table}'",
+        fetchone=True,
+    )
+    if not exist or not exist[0]:
+        return False
+    return True
+
+
 def _execute_sql(
     connection,
     sql,
@@ -857,6 +869,9 @@ def remove_webassets(conn):
         f"delete from ir_attachment where name ilike 'web_icon_data' {ignore_url_str}",
         f"delete from ir_attachment where name ilike 'web_editor.summernote.%' {ignore_url_str}",
         f"delete from ir_attachment where name ilike 'web.assets_backend_prod_only.js'",
+        # following is like odoo 16:
+        f"delete from ir_attachment where name ilike '%.assets_%.css' and res_model = 'ir.ui.view'",
+        f"delete from ir_attachment where name ilike '%.assets_%.js' and res_model = 'ir.ui.view'",
     ]
     try:
         for query in queries:
@@ -881,13 +896,6 @@ def get_dockercompose():
     return compose
 
 
-def get_volume_names():
-    from . import project_name
-
-    vols = get_dockercompose()["volumes"].keys()
-    return [f"{project_name}_{x}" for x in vols]
-
-
 def __running_as_root_or_sudo():
     output = subprocess.check_output(["/usr/bin/id", "-u"]).strip().decode("utf-8")
     return output == "0"
@@ -941,6 +949,16 @@ def __hash_odoo_password(pwd):
     ]:
         setpw = CryptContext(schemes=["pbkdf2_sha512", "md5_crypt"])
         return setpw.encrypt(pwd)
+    elif current_version() in [
+        17.0,
+    ]:
+        MIN_ROUNDS = 600_000
+        setpw = CryptContext(
+            schemes=["pbkdf2_sha512", "plaintext"],
+            deprecated=["auto"],
+            pbkdf2_sha512__rounds=max(MIN_ROUNDS, 0),
+        )
+        return setpw.encrypt(pwd)
     else:
         raise NotImplementedError()
 
@@ -951,6 +969,8 @@ def abort(msg, nr=1):
 
 
 def sync_folder(dir, dest_dir, excludes=None):
+    import platform
+
     dir = Path(dir)
     dest_dir = Path(dest_dir)
     dest_dir.mkdir(exist_ok=True, parents=True)
@@ -1143,9 +1163,20 @@ def get_hash(text):
     return hashlib.sha1(text).hexdigest()
 
 
+def remove_pycs(path):
+    path = path.absolute()
+    if not path.is_dir():
+        return
+    for cachedir in bashfind(path, name="__pycache__", type="d"):
+        shutil.rmtree(cachedir)
+    for file in bashfind(path, name="*.pyc", type="f"):
+        file.unlink()
+
+
 def get_directory_hash(path):
     click.secho(f"Calculating hash for {path}", fg="yellow")
     # "-N required because absolute path is used"
+    remove_pycs(path)
     hex = (
         subprocess.check_output(
             ["dtreetrawl", "-N", "--hash", "-R", path], encoding="utf8"
@@ -1413,39 +1444,48 @@ def _write_file(file, content):
 
 def _make_sure_module_is_installed(ctx, config, modulename, repo_url):
     from .module_tools import DBModules
+    from .module_tools import Modules, Module
     from .odoo_config import MANIFEST
     from .odoo_config import current_version
+    from .module_tools import NotInAddonsPath
     from .cli import cli, pass_config, Commands
     from gimera.gimera import add as gimera_add
     from gimera.gimera import apply as gimera_apply
 
     state = DBModules.get_meta_data(modulename)
-    if state["state"] == "installed":
+    if state and state["state"] == "installed":
         return
+    import pudb
 
-    path = Path("addons_wodoo") / modulename
-    if not path.exists():
-        ctx.invoke(
-            gimera_add,
-            url=repo_url,
-            path=str(path),
-            branch=str(current_version()),
-            type="integrated",
-        )
-        ctx.invoke(gimera_apply, repos=str(path))
+    pudb.set_trace()
 
-    # if not yet there, then pack into "addons_framework"
     manifest = MANIFEST()
-    addons_paths = manifest.get("addons_paths", [])
+    try:
+        Module.get_by_name(modulename)
+    except (KeyError, NotInAddonsPath):
+        path = Path("addons_wodoo") / modulename
+        if not path.exists():
+            ctx.invoke(
+                gimera_add,
+                url=repo_url,
+                path=str(path),
+                branch=str(current_version()),
+                type="integrated",
+            )
+            ctx.invoke(gimera_apply, repos=[str(path)])
+
+            # if not yet there, then pack into "addons_framework"
+            addons_paths = manifest.get("addons_paths", [])
+
+            if str(path) not in addons_paths:
+                addons_paths += [str(path)]
+                manifest["addons_paths"] = addons_paths
+
     install = manifest.get("install", [])
     if modulename not in install:
         install += [modulename]
     manifest["install"] = install
 
-    if str(path) not in addons_paths:
-        addons_paths += [str(path)]
-        manifest["addons_paths"] = addons_paths
-
     manifest.rewrite()
 
     Commands.invoke(
@@ -1459,30 +1499,39 @@ def _make_sure_module_is_installed(ctx, config, modulename, repo_url):
     )
 
 
-def bashfind(path, name=None, wholename=None, type=None):
-    cmd = [
-        "find",
-    ]
+def bashfind(path, name=None, wholename=None, type=None, maxdepth=None):
+    cmd = ["find", path]
     if type:
-        cmd += [
-            "-type",
-            type,
-        ]
+        cmd += ["-type", type]
+    if maxdepth:
+        cmd += ["-maxdepth", str(maxdepth)]
     if wholename:
         cmd += ["-wholename", wholename]
     if name:
         cmd += ["-name", name]
-    files = subprocess.check_output(cmd, cwd=path, encoding="utf8").splitlines()
-    return map(lambda x: Path(path) / x, files)
+    if not Path(path).exists():
+        return []
+    files = subprocess.check_output(cmd, encoding="utf8").splitlines()
+    return list(map(lambda x: Path(path) / x, files))
 
 
 def _update_setting(conn, key, value):
     value = str(value)
-    _execute_sql(
+    row = _execute_sql(
         conn,
-        f"DELETE FROM ir_config_parameter WHERE key = '{key}'; "
-        f"INSERT INTO ir_config_parameter(key, value, create_date, write_date) values('{key}', '{value}', now(), now());",
+        f"SELECT value FROM ir_config_parameter WHERE key = '{key}'",
+        fetchone=True,
     )
+    if not row:
+        _execute_sql(
+            conn,
+            f"INSERT INTO ir_config_parameter(key, value, create_date, write_date) values('{key}', '{value}', now(), now());",
+        )
+    else:
+        _execute_sql(
+            conn,
+            f"UPDATE ir_config_parameter SET value = '{value}' where key = '{key}'",
+        )
 
 
 def _get_setting(conn, key):
@@ -1523,23 +1572,6 @@ def atomic_write(file):
                 pass
 
 
-def bash_find(path, name=None, wholename=None, type=None):
-    cmd = [
-        "find",
-    ]
-    if type:
-        cmd += [
-            "-type",
-            type,
-        ]
-    if wholename:
-        cmd += ["-wholename", wholename]
-    if name:
-        cmd += ["-name", name]
-    files = subprocess.check_output(cmd, cwd=path, encoding="utf8").splitlines()
-    return list(map(lambda x: Path(path) / x, files))
-
-
 def pretty_xml(xmlstring):
     with autocleanpaper() as tempfile:
         tempfile.write_bytes(xmlstring)
@@ -1550,3 +1582,51 @@ def pretty_xml(xmlstring):
             },
         )
         return formatted
+
+
+def __get_postgres_volume_name(config):
+    if config.NAMED_ODOO_POSTGRES_VOLUME:
+        return config.NAMED_ODOO_POSTGRES_VOLUME
+    return f"{config.project_name}_odoo_postgres_volume"
+
+
+def get_volume_fullpath(name):
+    return get_docker_volumes() / name
+
+
+def get_docker_volumes():
+    DOCKER_VOLUMES = Path("/var/lib/docker/volumes")
+    return DOCKER_VOLUMES
+
+
+def get_directory_size(path):
+    size = subprocess.check_output(["du", "-s", path], encoding="utf8")
+    size = size.splitlines()[-1].split()[0]
+    return int(size)
+
+
+def _get_xml_id(config, model, id):
+    conn = config.get_odoo_conn()
+    rows = _execute_sql(
+        conn,
+        f"select module, name from ir_model_data where res_id={id} and model='{model}'",
+        fetchall=True,
+    )
+    if rows:
+        return f"{rows[0][0]}.{rows[0][1]}"
+
+
+def print_prod_env(config):
+    if config.devmode:
+        return
+    click.secho("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!", fg="magenta")
+    click.secho("        PRODUCTION ENVIRONMENT      ", fg="magenta")
+    click.secho("!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!", fg="magenta")
+    time.sleep(2)
+
+
+def force_input_hostname():
+    hostname = socket.gethostname()
+    value = click.prompt(f"Please type the hostname of the machine again [{hostname}]")
+    if value != hostname:
+        abort(f"You typed {value} but {hostname} was expected.")
